% Simulating the 2-D Diffusion equation by the Finite Difference
...Method 
% Numerical scheme used is a first order upwind in time and a second
...order central difference in space (Implicit and Explicit)
%%
clear; clc; close all;
% New colormap (magenta)
lim = 1000; cmm = [ones(1,lim); 1-((1:lim)/lim).^2./(0.2^2+((1:lim)./lim).^2); ones(1,lim)]';
%Specifying parameters
Nx  = 40;                  %Number of steps in space(x)
Ny  = 40;                  %Number of steps in space(y)       
M   = 40;                  %Number of time steps 
dt  = 0.5;                 %Width of each time step
Lx  = 2;                   %Space length (x direction)
Ly  = 2;                   %Space length (y direction)
dx  = Lx/(Nx-1);           %Width of space step(x)
dy  = Ly/(Ny-1);           %Width of space step(y)
x   = 0:dx:Lx;             %Range of x(0,Lx) and specifying the grid points
y   = 0:dy:Ly;             %Range of y(0,Ly) and specifying the grid points
u   = zeros(Nx,Ny);        %Concentration u
um  = zeros(Nx,Ny);        %Concentration u^m (previous time level)
D   = 0.01;                %Diffusion coefficient
UW  = 0;                   %x=0 Dirichlet B.C 
UE  = 0;                   %x=L Dirichlet B.C 
US  = 0;                   %y=0 Dirichlet B.C 
UN  = 0;                   %y=L Dirichlet B.C  
UnW = 0;                   %x=0 Neumann B.C (du/dn=UnW)
UnE = 0;                   %x=L Neumann B.C (du/dn=UnE)
UnS = 0;                   %y=0 Neumann B.C (du/dn=UnS)
UnN = 0;                   %y=L Neumann B.C (du/dn=UnN)
%%
%Initial Conditions
for i = 1:Nx
    for j = 1:Ny
        L = 100; u(i,j) = exp(-(i-Nx/2)^2/L-(j-Ny/2)^2/L);
    end
end
%%
%B.C vector
bc = zeros(Nx-2,Ny-2);
%bc(1,:) = UW/dx^2; bc(Nx-2,:) = UE/dx^2;       %Dirichlet B.Cs
%bc(:,1) = US/dy^2; bc(:,Ny-2) = UN/dy^2;       %Dirichlet B.Cs
bc(1,:) = -UnW/dx; bc(Nx-2,:) = UnE/dx;        %Neumann B.Cs
bc(:,1) = -UnS/dy; bc(:,Ny-2) = UnN/dy;        %Neumann B.Cs
%B.Cs at the corners:
%bc(1,1)    = UW/dx^2+US/dy^2; bc(Nx-2,1)    = UE/dx^2+US/dy^2;
%bc(1,Ny-2) = UW/dx^2+UN/dy^2; bc(Nx-2,Ny-2) = UE/dx^2+UN/dy^2;
bc = D*dt*bc;

%Calculating the coefficient matrix 
Ex = sparse(2:Nx-2,1:Nx-3,1,Nx-2,Nx-2);
Ax = Ex+Ex' - 2*speye(Nx-2);       
Ax(1,1) = -1; Ax(Nx-2,Nx-2) = -1;        %Neumann B.Cs (comment for Dirichlet B.Cs)
Ey = sparse(2:Ny-2,1:Ny-3,1,Ny-2,Ny-2);
Ay = Ey+Ey' - 2*speye(Ny-2);       
Ay(1,1) = -1; Ay(Ny-2,Ny-2) = -1;        %Neumann B.Cs (comment for Dirichlet B.Cs)
A = kron(Ay/dy^2,speye(Nx-2)) + kron(speye(Ny-2),Ax/dx^2);
B = speye((Nx-2)*(Ny-2)) - D*dt*A; % Matrix for the implicit scheme

%%
%Calculating the field variable for each time step
i = 2:Nx-1;
j = 2:Ny-1;
for m = 0:M
    um = u;
    % 2D surface
    h = pcolor(x,y,u');
    title({['2-D Diffusion with D = ',num2str(D)];['time (\itt) = ',num2str(m*dt)]})
    shading interp
    caxis([0 1])
    colormap(cmm)
    xlabel('Spatial co-ordinate (x)')
    ylabel('Spatial co-ordinate (y)')
    drawnow; 
    refreshdata(h)
    U = um; U(1,:)=[]; U(end,:)=[]; U(:,1)=[]; U(:,end)=[];
    U = reshape(U+bc,[],1);
    U = B\U;             %Implicit method
    %U = U + D*dt*A*U;   %Explicit method
    U = reshape(U,Nx-2,Ny-2);
    u(2:Nx-1,2:Ny-1) = U;
    %Boundary conditions
    %Dirichlet:
    %u(1,:)  = UW; u(Nx,:) = UE;
    %u(:,1)  = US; u(:,Ny) = UN;
    %Neumann:
    u(1,:) = u(2,:) - UnW*dx; u(Nx,:) = u(Nx-1,:) + UnE*dx;
    u(:,1) = u(:,2) - UnS*dy; u(:,Ny) = u(:,Ny-1) + UnN*dy;
end