% Simulating the 2-D Diffusion equation (Fourier's equation) by the Finite
... with Difference Method (Dirichlet boundary conditions)
% Numerical scheme used is a first order explicit upwind in time and a 
... second order central difference in space
%%
clear; clc; close all;
% New colormap (magenta)
lim = 1000; cmm=[ones(1,lim); 1-((1:lim)/lim).^2./(0.2^2+((1:lim)./lim).^2); ones(1,lim)]';
%Specifying parameters
Nx = 40;                   %Number of steps in space(x)
Ny = 40;                   %Number of steps in space(y)       
M = 40;                    %Number of time steps 
dt = 0.05;                 %Width of each time step
Lx = 2;                    %Space length (x direction)
Ly = 2;                    %Space length (y direction)
dx = Lx/(Nx-1);            %Width of space step(x)
dy = Ly/(Ny-1);            %Width of space step(y)
x = 0:dx:Lx;               %Range of x(0,Lx) and specifying the grid points
y = 0:dy:Ly;               %Range of y(0,Ly) and specifying the grid points
u = zeros(Nx,Ny);          %Concentration u
um = zeros(Nx,Ny);         %Concentration u^m (previous time level)
D = 0.01;                  %Diffusion coefficient
UW = 0;                    %x=0 Dirichlet B.C 
UE = 0;                    %x=L Dirichlet B.C 
US = 0;                    %y=0 Dirichlet B.C 
UN = 0;                    %y=L Dirichlet B.C 
%%
%Initial Conditions
for i = 1:Nx
    for j = 1:Ny
        L = 10; u(i,j) = exp(-(i-Nx/2)^2/L-(j-Ny/2)^2/L);
    end
end
%%
%Calculating the field variable for each time step
i = 2:Nx-1;
j = 2:Ny-1;
for m = 0:M
    um = u;
    % 3D surface
    subplot(211), h=surf(x,y,u','EdgeColor','none'); colormap(cmm);    
    shading interp
    axis ([0 Lx 0 Ly 0 1.5])
    title({['2-D Diffusion with D = ',num2str(D)];['time (\itt) = ',num2str(m*dt)]})
    xlabel('Spatial co-ordinate (x)')
    ylabel('Spatial co-ordinate (y)')
    zlabel('Concentration profile (u)')
    drawnow; 
    refreshdata(h)
    % 2D plot
    subplot(212), pcolor(x,y,u');
    shading interp
    caxis([0 1])
    xlabel('Spatial co-ordinate (x)')
    ylabel('Spatial co-ordinate (y)')
    %Boundary conditions
    u(1,:) = UW; u(Nx,:) = UE;
    u(:,1) = US; u(:,Ny) = UN;
    %Explicit method:
    u(i,j) = um(i,j) + (D*dt*(um(i+1,j) - 2*um(i,j) + um(i-1,j))/(dx*dx)) +...
                       (D*dt*(um(i,j+1) - 2*um(i,j) + um(i,j-1))/(dy*dy));
end